<?php
include_once(realpath(dirname(__FILE__)) . "/../../../classes/PluginsClassiques.class.php");
include_once(realpath(dirname(__FILE__)) . "/../../../classes/Variable.class.php");

class Multisquelettes extends PluginsClassiques
{
	const prefix = 'ms_';

	const MODULE    = 'multisquelettes';
	const NOMMODULE = 'Multi Squelettes';
	const VERSION   = '1.3.5';
	const PREFIXE   = 'ms_';

	const REP_THELIA_DEFAUT = 'template';

	var $squelette;

	public function Multisquelettes()
	{
		parent::PluginsClassiques(self::MODULE);

		$this->debug_actif = intval($this->get_variable('ms_debug')) != 0;

		$this->trouver_rep_squelette();
	}

	function init()
    {
		$this->ajout_desc(
                self::NOMMODULE,
                "Plugin Squelettes Multiples",
                "Ce plugin permet d'utiliser plusieurs jeux de squelettes sur votre Thelia",
                1);

        $this->create_variable('ms_basedir', '');
        $this->create_variable('ms_activer', false);
        $this->create_variable('ms_defaut', self::REP_THELIA_DEFAUT);
        $this->create_variable('ms_common', self::REP_THELIA_DEFAUT);
        $this->create_variable('ms_mobile', self::REP_THELIA_DEFAUT);
        $this->create_variable('ms_actifs', '');
        $this->create_variable('ms_debug', false);
    }

    function destroy()
    {
        $this->delete_variable('ms_basedir');
        $this->delete_variable('ms_activer');
        $this->delete_variable('ms_defaut');
        $this->delete_variable('ms_common');
        $this->delete_variable('ms_mobile');
        $this->delete_variable('ms_actifs');
        $this->delete_variable('ms_debug');
    }

    function boucle($texte, $args)
    {
    	$res = '';

    	$courant = intval(lireTag($args, 'courant'));
    	$defaut = intval(lireTag($args, 'defaut'));

    	if ($this->squelette !== false)
    	{
    		$def = $this->get_default_template();

			$squelettes = explode(',', $this->get_variable('ms_actifs'));

			foreach($squelettes as $sq)
			{
				if ($courant != 0 && $sq != $_SESSION['squelette']) continue;

				if ($defaut != 0 && $sq != $def) continue;

				$tmp = str_replace('#SQUELETTE', $sq, $texte);

				$tmp = str_replace('#COURANT', $sq == $_SESSION['squelette'] ? '1' : '0', $tmp);
				$tmp = str_replace('#DEFAUT', $sq == $def ? '1' : '0', $tmp);

				$res .= $tmp;
			}
		}

		return $res;
    }

    private function get_template($file, $pour_inclure = false)
    {
    	$this->_debug("Rercherche du fichier ", $file, ($pour_inclure != false ? ' pour #INCLURE ou #SQUELETTE_TROUVER' : ''));

    	// Traiter un #SQUELETTE_TROUVER éventuel
		$file = $this->squelette_trouver($file);

    	// Normaliser le fichier (on retire le repertoire par defaut de Thelia)
    	$file = preg_replace('!^'.self::REP_THELIA_DEFAUT . '/!', "", $file);

    	$this->_debug("Après normalisation et substitution, fichier: $file");

    	if ($this->squelette == self::REP_THELIA_DEFAUT) {

    		$this->_debug("Le squelette courant est le squelette par défaut (", self::REP_THELIA_DEFAUT, ") -> terminé.");

    		return self::REP_THELIA_DEFAUT . "/$file";
    	}

    	$tpl = $this->squelette . '/' . $file ;

    	if ($pour_inclure == false) $tpl .=  '.html';

    	$this->_debug("Test de l'existence du fichier $tpl");

		// Fichier non trouve pour ce squelette ? Aller le chercher dans
		// le répertoire des fichiers communs.
		if (! file_exists($tpl))
		{
			$def = $this->get_variable('ms_common');

			$this->_debug("Fichier $tpl inexistant, utilisation du fichier commun dans ($def)");

			if ($def != '' && $def != self::REP_THELIA_DEFAUT)
				$tpl = $this->get_variable('ms_basedir') . '/' . $def . '/' . $file;
			else
				$tpl = "$def/$file";

			// Le fichier n'existe toujours pas ? Utiliser le chemoin indiqué
			if ($pour_inclure == false) $tpl .=  '.html';

			if (! file_exists($tpl)) {

				$this->_debug("Fichier $tpl inexistant, utilisation du nom tel qu'indiqué");

				$tpl = $file;
			}
		}

		$this->_debug("Template retourné: ", $tpl);

		return $tpl;
    }

    // Modifier la variable 'fond' pour y placer le répertoire du squelette, et eventuellement
    // le répertoire
    public function pre()
    {
    	global $fond, $reptpl, $sortie, $lang;

    	// Si la variable 'sortie' est à false, on est sans doute en
    	// train d'essayer de créer un PDF -> ne pas tenter de modifier
    	// le repertoire des templates.
    	if ($this->squelette !== false && $sortie)
    	{
    		$tpl = $this->get_template($fond);

    		$fond = basename($tpl);
    		$reptpl = rtrim(dirname($tpl), '/') . '/';

			$this->_debug("fond=$fond");
			$this->_debug("reptpl=$reptpl");


			// Trouver le fichier de langue (on essaie d'abord <code>.php puis <id>.php
			$this->_debug("Recherche du fichier de langue à utiliser");

			$langobj = new Lang($lang);

			$lang_file = $this->get_template("lang/".strtolower($langobj->code).".php", true);

			if (! file_exists($lang_file))
				$lang_file = $this->get_template("lang/".$langobj->id.".php", true);

			$this->_debug("Fichier de langue utilisé: $lang_file");

			require_once($lang_file);
    	}
    }

    private function traiter_inclusion(&$texte)
    {
    	$res = $texte;

     	if (preg_match_all('/#INCLURE[\s]*"([^"]+)"/', $res, $matches, PREG_SET_ORDER))
    	{
    		foreach($matches as $match)
    		{
				$this->_debug("Inclusion du fichier ",$match[1]);

    			$inc = $this->get_template($match[1], true);

				$this->_debug("Fichier inclus trouvé: ", $inc);

				if (file_exists($inc))
				{
					$inctexte = file_get_contents($inc);

    				$res = str_replace($match[0], $this->traiter_inclusion($inctexte), $res);
				}
				else
				{
					die ("Impossible d'ouvrir le fichier inclus $inc");
				}
    		}
    	}

    	return $res;
    }

    /*
     * Open source mobile phone detection
     * http://detectmobilebrowser.com/
     */
    private $is_mobile = -1;

    private function is_mobile_client() {

    	if ($this->is_mobile == -1) {
			$useragent = $_SERVER['HTTP_USER_AGENT'];

			$this->is_mobile = preg_match('/android|avantgo|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i',$useragent)||preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|e\-|e\/|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(di|rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|xda(\-|2|g)|yas\-|your|zeto|zte\-/i',substr($useragent,0,4)) > 0;

			if ($this->is_mobile) $this->_debug("navigateur mobile détecté (user-agent: $useragent)");
    	}

    	return $this->is_mobile;
    }

    private function get_default_template() {
    	if ($this->is_mobile_client())
    		return $this->get_variable('ms_mobile');
    	else
    		return $this->get_variable('ms_defaut');
    }

    // Modifier les includes dans $res pour y ajouter le répertoire du squelette
    public function inclusion()
    {
    	if ($this->squelette !== false)
    	{
		    global $res;

		    $res = $this->traiter_inclusion($res);
    	}
    }

    public function action()
    {
        global $res;

    	if (strstr($res, '#SQUELETTE_') === false) return;

    	if ($this->squelette !== false)
        {
    		$res = str_replace('#SQUELETTE_COURANT', basename($this->squelette), $res);
        }
        else
        {
     		$res = str_replace('#SQUELETTE_COURANT', '', $res);
        }

		$res = str_replace('#SQUELETTE_BASE'  , $this->get_variable('ms_basedir'), $res);
		$res = str_replace('#SQUELETTE_DEFAUT', $this->get_default_template() , $res);
		$res = str_replace('#SQUELETTE_DEFAUT_MOBILE' , $this->get_variable('ms_mobile') , $res);
		$res = str_replace('#SQUELETTE_DEFAUT_DESKTOP', $this->get_variable('ms_defaut') , $res);
		$res = str_replace('#SQUELETTE_COMMUN', $this->get_variable('ms_common') , $res);
    }

    public function post()
    {
    	global $res;

    	if (strstr($res, '#SQUELETTE_') === false) return;

    	if ($this->squelette !== false)
    	{
    		$res = $this->squelette_trouver($res);
    	}
    	else
    	{
    		$res = preg_replace('/\#SQUELETTE_TROUVER\(([^\(]+)\)/', "\\1", $res);

    		$res = preg_replace('/\#SQUELETTE_CHEMIN[\/]*/', '', $res);
    	}
    }

    private function trouver_rep_squelette()
    {
    	$defaut = $this->get_default_template();

    	if (intval($this->get_variable('ms_activer')) == 0)
    	{
    		$this->squelette = false;

    		return;
    	}

    	if (isset($_REQUEST['squelette']))
    	{
    		if ($_REQUEST['squelette'] == '')
    		{
    			$squelette = $defaut;
    		}
    		else
    		{
    			// Par sécurité, on ne prend que le basename (ce qu'il y a après le dernier '/',
    			// afin d'éviter des hacks du genre ../../../nom_dir_secret
    			$squelette = basename(strip_tags($_REQUEST['squelette']));
    		}
    	}
    	else if (isset($_SESSION['squelette']))
    	{
    		$squelette = $_SESSION['squelette'];
    	}
    	else
    	{
    		$squelette = $defaut;
    	}

    	if ($squelette != '')
    	{
    		if (strstr($this->get_variable('ms_actifs'), $squelette) === false) $squelette = $defaut;

    		$this->squelette = $squelette != self::REP_THELIA_DEFAUT ? $this->get_variable('ms_basedir') . '/' . $squelette : self::REP_THELIA_DEFAUT;
    	}

    	// Si le squelette choisi est le squelette par défaut, on ne stocke rien en session
    	if ($squelette == $defaut && isset($_REQUEST['squelette']))
    		unset($_SESSION['squelette']);
    	else
    		$_SESSION['squelette'] = $squelette;
    }

    //-- Outils ----------------------------------------------------------------

    private function squelette_trouver($str) {

    	if (preg_match_all('/\#SQUELETTE_TROUVER\(([^\)]+)\)/', $str, $matches, PREG_SET_ORDER)) {

			foreach($matches as $match) {

				$this->_debug("#SQUELETTE_TROUVER(".$match[1].")");

    			$str = str_replace($match[0], $this->get_template($match[1], true), $str);
			}
    	}

    	$str = preg_replace('/\#SQUELETTE_CHEMIN[\/]*/', $this->squelette . ($this->squelette != '' ? '/' : ''), $str);

    	return $str;
    }

	private function _debug()
	{
		if (! $this->debug_actif) return;

		$text = '';

		$numargs = func_num_args();

			for($idx = 0; $idx < $numargs; $idx++)
			{
				$arg = func_get_arg($idx);

					$text .= is_scalar($arg) ? $arg : print_r($arg, true);
			}

			echo '<pre style="text-align: left; margin: 0; padding: 0;">[DEBUG] ' . htmlspecialchars($text)."</pre>";
	}

    public function get_variable($nom)
    {
        $var = new Variable();
        if ($var->charger($nom))
        {
            return $var->valeur;
        }

        return false;
    }

    protected function create_variable($nom, $valeur)
    {
        $variable = new Variable();

        $variable->nom = $nom;
        $variable->valeur = $valeur;
        $variable->protege = 1;
        $variable->cache = 1;

        $variable->add();
    }

    protected function delete_variable($nom)
    {
        $variable = new Variable();

        if ($variable->charger($nom)) $variable->delete();
    }

}
?>
